#!/bin/sh
# 
# References: 1. Sequence-depent Correction of Random Coil NMR
#             Chemical Shifts
#             (JACS, 2001, 123, 2970-2978)
#             2. Random coil chemical shifts in acidic 8M urea:
#             Implementation of random coil shift data in NMRView
#             (JBNMR, 2000, 18, 43-48)
#			  3. 1H, 13C and 15N random coil NMR chemical shifts of the common amino acids.
#				 I. Investigations of nearest-neighbor effects
#			  (JBNMR, 1995, 5, 67-81)
#
#			  Author : Raghavendran Lakshmi Narayanan, NMR-based structural biology, MPIBPC, goettingen.
#			  Modifications : corrected sequence dependency bugs, replaced schwarzinger's values 
#				by wisharts for carbon chemical shifts of charged residues, All Nitrogen chemical
#				shifts are from wisharts values also includes preproline nitrogen chemical shifts.
#			  This shell script is modified form the following source. 
#              Author Kuen-Phon Wu, Dept, Rutgers University
#                     May 19th, 2006
#                     update : Jun. 2nd, 2006
# 		      fix shell issues  for mac os x 2007,07,11

#### the sequence files should not have any comments 
##### accepts SEQuence1L.tab from mars run


### deal with CO chemical shift -- modified A (residue+2) column to match NMRvj8 prediction 
### will output a temp file for corrected random coil CO shifts 
tr 'A-Z' 'a-z' < $1 | tr '[:punct:]' 'b' | tr '0-9' 'b' |tr '\n' 'b'| \
sed -e 's/[o|x|z|u|j|b| ]//g' \
-e 's/a/A     178.47    0.0    -0.77    -0.07    -0.02\n/g' \
-e 's/c/C     175.30    0.00    -0.51    -0.28    -0.07\n/g' \
-e 's/d/D     176.30    0.00    -0.58    -0.13    -0.04\n/g' \
-e 's/e/E     176.60    0.00    -0.48    -0.20    -0.03\n/g' \
-e 's/f/F     176.62    0.00    -0.83    -0.25    -0.10\n/g' \
-e 's/g/G     174.93    0.00     0.00     0.00     0.00\n/g' \
-e 's/h/H     174.10    0.00    -0.65    -0.22    -0.07\n/g' \
-e 's/i/I     177.12    0.00    -0.58    -0.18    -0.02\n/g' \
-e 's/k/K     176.60    0.00    -0.50    -0.18    -0.03\n/g' \
-e 's/l/L     178.23    0.00    -0.50    -0.13    -0.01\n/g' \
-e 's/m/M     177.06    0.00    -0.41    -0.18    -0.02\n/g' \
-e 's/p/P     177.81    -0.47    -2.84    -0.09    -0.02\n/g' \
-e 's/r/R     176.30    0.00    -0.49    -0.19    -0.03\n/g' \
-e 's/q/Q     176.77    0.00    -0.48    -0.18    -0.03\n/g' \
-e 's/n/N     176.07    0.00    -0.66    -0.10    -0.03\n/g' \
-e 's/s/S     175.38    0.00    -0.40    -0.15    -0.06\n/g' \
-e 's/t/T     175.60    0.00    -0.19    -0.13    -0.05\n/g' \
-e 's/w/W     177.05    0.00    -0.85    -0.30    -0.17\n/g' \
-e 's/y/Y     176.71    0.00    -0.85    -0.24    -0.13\n/g' \
-e 's/v/V     177.04    0.00    -0.57    -0.18    -0.03\n/g' |awk '{gsub("n","\n");print}'> _coil

###  calculation residue numbers $i
i=`wc -l _coil | awk '{print $1 }'` 
j=`expr $i - 1`
k=`expr $i - 2`

echo 0.00 >_temp_4
#echo "0.00" >>_temp_5 | echo 0.00 >>_temp_5

cat _coil | awk '{ print $1, $2}' > _temp_1
cat _coil | awk '{ print $3}' | tail -n $k > _temp_2
cat _coil | awk '{ print $4}' | tail -n $j > _temp_3
cat _coil | awk '{ print $5}' | head -n $j >> _temp_4
#cat _coil | awk '{ print $6}' | head -n $k > _temp_5
echo "0.00" >>_temp_2 | echo 0.00 >> _temp_2
echo 0.00 >>_temp_3 
paste _temp_* | sed -e 's/ /      /g' | awk '{ print $2+$3+$4+$5+$6}' >_co_corrections


rm _*temp_*

### deal with CA chemical shift 
### will output a temp file for corrected random coil CA shifts 
tr 'A-Z' 'a-z' < $1 | tr '[:punct:]' 'b' | tr '0-9' 'b' |tr '\n' 'b'| \
sed -e 's/[o|x|z|u|j|b| ]//g' \
-e 's/a/A     52.82    -0.02    -0.17     0.06     0.01\n/g' \
-e 's/c/C     58.63    -0.03    -0.07     0.10    -0.01\n/g' \
-e 's/d/D     54.20    -0.03     0.00     0.25    -0.01\n/g' \
-e 's/e/E     56.60    -0.01    -0.08     0.05     0.01\n/g' \
-e 's/f/F     58.09    -0.07    -0.23     0.06     0.01\n/g' \
-e 's/g/G     45.39     0.00     0.00     0.00     0.00\n/g' \
-e 's/h/H     55.00    -0.05    -0.09     0.02     0.01\n/g' \
-e 's/i/I     61.62    -0.07    -0.20    -0.01     0.02\n/g' \
-e 's/k/K     56.20    -0.01    -0.11    -0.02     0.02\n/g' \
-e 's/l/L     55.47    -0.01    -0.10     0.03     0.02\n/g' \
-e 's/m/M     55.77     0.00     0.10    -0.06     0.01\n/g' \
-e 's/n/N     53.33    -0.06    -0.03     0.23     0.01\n/g' \
-e 's/p/P     63.70    -0.22    -2.00     0.02     0.04\n/g' \
-e 's/q/Q     56.22    -0.02    -0.06     0.04     0.01\n/g' \
-e 's/r/R     56.00     0.00    -0.07    -0.01     0.02\n/g' \
-e 's/s/S     58.67     0.00    -0.08     0.13     0.00\n/g' \
-e 's/t/T     62.01    -0.01    -0.04     0.12     0.00\n/g' \
-e 's/v/V     62.61    -0.07    -0.21    -0.02     0.01\n/g' \
-e 's/w/W     57.60    -0.02    -0.17     0.03    -0.08\n/g' \
-e 's/y/Y     58.28    -0.07    -0.22     0.06    -0.01\n/g' |awk '{gsub("n","\n");print}' > _coil

echo 0.00 >_temp_4
#echo "0.00" >>_temp_5 | echo 0.00 >>_temp_5

cat _coil | awk '{ print $1, $2}' > _temp_1
#cat _coil | awk '{ print $3}' | tail -n $k >> _temp_2
cat _coil | awk '{ print $4}' | tail -n $j > _temp_3
cat _coil | awk '{ print $5}' | head -n $j >> _temp_4
#cat _coil | awk '{ print $6}' | head -n $k > _temp_5
#echo "0.00" >_temp_2 | echo 0.00 >> _temp_2
echo 0.00 >>_temp_3 
paste _temp_* | sed -e 's/ /      /g' | awk '{ print $2+$3+$4+$5+$6}' >_ca_corrections

rm _*temp_*

### deal with HA chemical shift 
### will output a temp file for corrected random coil HA shifts 
tr 'A-Z' 'a-z' < $1 | tr '[:punct:]' 'b' | tr '0-9' 'b' |tr '\n' 'b'| \
sed -e 's/[o|x|z|u|j|b| ]//g' \
-e 's/a/A     4.35    -0.02    -0.03    -0.03     0.00\n/g' \
-e 's/c/C     4.59    -0.01     0.02     0.00     0.00\n/g' \
-e 's/d/D     4.82    -0.02    -0.01    -0.02    -0.01\n/g' \
-e 's/e/E     4.42    -0.02    -0.02    -0.02     0.00\n/g' \
-e 's/f/F     4.65    -0.06    -0.09    -0.08    -0.04\n/g' \
-e 's/g/G     4.02     0.00     0.00     0.00     0.00\n/g' \
-e 's/h/H     4.79    -0.03    -0.06     0.01     0.01\n/g' \
-e 's/i/I     4.21    -0.03    -0.02    -0.02    -0.01\n/g' \
-e 's/k/K     4.36    -0.02    -0.02    -0.01     0.00\n/g' \
-e 's/l/L     4.38    -0.04    -0.03    -0.05    -0.01\n/g' \
-e 's/m/M     4.52    -0.02    -0.01    -0.01     0.00\n/g' \
-e 's/n/N     4.79    -0.01    -0.01    -0.02    -0.01\n/g' \
-e 's/p/P     4.45    -0.01     0.11    -0.03    -0.01\n/g' \
-e 's/q/Q     4.38    -0.01    -0.02    -0.01     0.00\n/g' \
-e 's/r/R     4.38    -0.02    -0.02    -0.02     0.00\n/g' \
-e 's/s/S     4.51    -0.01     0.02     0.00    -0.01\n/g' \
-e 's/t/T     4.43    -0.01     0.05     0.00    -0.01\n/g' \
-e 's/v/V     4.16    -0.02    -0.01    -0.02    -0.01\n/g' \
-e 's/w/W     4.70    -0.08    -0.10    -0.15    -0.16\n/g' \
-e 's/y/Y     4.58    -0.05    -0.10    -0.08    -0.04\n/g' |awk '{gsub("n","\n");print}'> _coil

echo 0.00 >_temp_4
#echo "0.00" >>_temp_5 | echo 0.00 >>_temp_5

cat _coil | awk '{ print $1, $2}' > _temp_1
#cat _coil | awk '{ print $3}' | tail -n $k >> _temp_2
cat _coil | awk '{ print $4}' | tail -n $j > _temp_3
cat _coil | awk '{ print $5}' | head -n $j >> _temp_4
#cat _coil | awk '{ print $6}' | head -n $k > _temp_5
#echo "0.00" >_temp_2 | echo 0.00 >> _temp_2
echo 0.00 >>_temp_3 
paste _temp_* | sed -e 's/ /      /g' | awk '{ print $2+$3+$4+$5+$6}' >_ha_corrections

rm _*temp_*

### deal with HN chemical shift 
### will output a temp file for corrected random coil HN shifts 
tr 'A-Z' 'a-z' < $1 | tr '[:punct:]' 'b' | tr '0-9' 'b' |tr '\n' 'b'| \
sed -e 's/[o|x|z|u|j|b| ]//g' \
-e 's/a/A     8.35    -0.01    -0.05     0.07    -0.10\n/g' \
-e 's/c/C     8.44     0.00    -0.02     0.20    -0.07\n/g' \
-e 's/d/D     8.56    -0.02    -0.03     0.14    -0.11\n/g' \
-e 's/e/E     8.40    -0.01    -0.03     0.15    -0.07\n/g' \
-e 's/f/F     8.31    -0.03    -0.12     0.10    -0.37\n/g' \
-e 's/g/G     8.41     0.00     0.00     0.00     0.00\n/g' \
-e 's/h/H     8.56    -0.01    -0.04     0.20     0.00\n/g' \
-e 's/i/I     8.17    -0.01    -0.06     0.17    -0.09\n/g' \
-e 's/k/K     8.36     0.00    -0.03     0.14    -0.06\n/g' \
-e 's/l/L     8.28     0.00    -0.03     0.14    -0.08\n/g' \
-e 's/m/M     8.42     0.00    -0.02     0.15    -0.06\n/g' \
-e 's/n/N     8.51    -0.01    -0.03     0.13    -0.07\n/g' \
-e 's/p/P     -5.008    -0.04    -0.18     0.19    -0.12\n/g' \
-e 's/q/Q     8.44    -0.01    -0.02     0.15    -0.06\n/g' \
-e 's/r/R     8.39     0.00    -0.02     0.15    -0.06\n/g' \
-e 's/s/S     8.43     0.00    -0.03     0.16    -0.08\n/g' \
-e 's/t/T     8.25     0.01     0.00     0.14    -0.06\n/g' \
-e 's/v/V     8.16    -0.01    -0.05     0.17    -0.08\n/g' \
-e 's/w/W     8.22    -0.08    -0.13     0.04    -0.62\n/g' \
-e 's/y/Y     8.26    -0.04    -0.11     0.09    -0.42\n/g' |awk '{gsub("n","\n");print}'> _coil

echo 0.00 >_temp_4
#echo "0.00" >>_temp_5 | echo 0.00 >>_temp_5

cat _coil | awk '{ print $1, $2}' > _temp_1
#cat _coil | awk '{ print $3}' | tail -n $k >> _temp_2
cat _coil | awk '{ print $4}' | tail -n $j > _temp_3
cat _coil | awk '{ print $5}' | head -n $j >> _temp_4
#cat _coil | awk '{ print $6}' | head -n $k > _temp_5
#echo "0.00" >_temp_2 | echo 0.00 >> _temp_2
echo 0.00 >>_temp_3 
paste _temp_* | sed -e 's/ /      /g' | awk '{ print $2+$3+$4+$5+$6}' | sed -e 's/^-....8/9999/g' | awk '{ print $0}' >_hn_corrections

rm _*temp_*

### deal with 15N chemical shift from wisharts values
### will output a temp file for corrected random coil 15N shifts 
tr 'A-Z' 'a-z' < $1 | tr '[:punct:]' 'b' | tr '0-9' 'b' |tr '\n' 'b'| \
sed -e 's/[o|x|z|u|j|b| ]//g' \
-e 's/a/A     123.8    -0.12    -0.33    -0.57    -0.15\n/g' \
-e 's/c/C     118.8    -0.06    -0.26     3.07     0.00\n/g' \
-e 's/d/D     120.4    -0.12    -0.20     0.86    -0.29\n/g' \
-e 's/e/E     120.2    -0.06    -0.20     1.51    -0.12\n/g' \
-e 's/f/F     120.3    -0.18    -0.49     2.78    -0.46\n/g' \
-e 's/g/G     108.8     0.00     0.00     0.00     0.00\n/g' \
-e 's/h/H     118.2    -0.12    -0.55     1.68     0.17\n/g' \
-e 's/i/I     119.9    -0.18    -0.14     4.87     0.00\n/g' \
-e 's/k/K     120.4    -0.06    -0.20     1.57    -0.06\n/g' \
-e 's/l/L     121.8    -0.06    -0.14     1.05    -0.06\n/g' \
-e 's/m/M     119.6    -0.06    -0.20     1.57    -0.06\n/g' \
-e 's/n/N     118.7    -0.18    -0.26     0.87    -0.17\n/g' \
-e 's/p/P     -5.008    -0.18    -0.32     0.87    -0.17\n/g' \
-e 's/q/Q     119.8    -0.06    -0.14     1.62    -0.06\n/g' \
-e 's/r/R     120.5    -0.06    -0.14     1.62    -0.06\n/g' \
-e 's/s/S     115.7    -0.06    -0.03     2.55    -0.17\n/g' \
-e 's/t/T     113.6    -0.06    -0.03     2.78    -0.12\n/g' \
-e 's/v/V     119.2    -0.24    -0.14     4.34    -0.06\n/g' \
-e 's/w/W     121.3     0.00    -0.26     3.19    -0.64\n/g' \
-e 's/y/Y     120.3    -0.24    -0.43     3.01    -0.52\n/g' |awk '{gsub("n","\n");print}'> _coil

echo 0.00 >_temp_4
#echo "0.00" >>_temp_5 | echo 0.00 >>_temp_5

cat _coil | awk '{ print $1, $2}' > _tttemp_1
#cat _coil | awk '{ print $3}' | tail -n $k >> _temp_2
cat _coil | awk '{ print $4}' | tail -n $j > _temp_3
cat _coil | awk '{ print $5}' | head -n $j >> _temp_4
#cat _coil | awk '{ print $6}' | head -n $k > _temp_5
#echo "0.00" >_temp_2 | echo 0.00 >> _temp_2
echo 0.00 >>_temp_3 

### deal with 15N chemical shift for preprolines
### will output a temp file for sequence corrected random coil 15N shifts for preprolines
### mark all preproline residues

awk '{res[++i]=$1;line[i]=$0;if(i>1 && res[i]=="P"){t_line=("p")(line[i-1]);print t_line}else{t_line=line[i-1];print t_line}}' _tttemp_1 > _ttemp_1

### replace marked preproline residues with preproline values
sed -e '/^$/d' \
-e 's/pA 123.8/A 125.0/g' \
-e 's/pC 118.8/C 119.9/g' \
-e 's/pD 120.4/D 121.4/g' \
-e 's/pE 120.2/E 121.7/g' \
-e 's/pF 120.3/F 129.9/g' \
-e 's/pG 108.8/G 109.1/g' \
-e 's/pH 118.2/H 118.2/g' \
-e 's/pI 119.9/I 121.7/g' \
-e 's/pK 120.4/K 121.6/g' \
-e 's/pL 121.8/L 122.6/g' \
-e 's/pM 119.6/M 120.7/g' \
-e 's/pN 118.7/N 119.0/g' \
-e 's/pP -5.008/P -5.008/g' \
-e 's/pQ 119.8/Q 120.6/g' \
-e 's/pR 120.5/R 121.3/g' \
-e 's/pS 115.7/S 116.6/g' \
-e 's/pT 113.6/T 116.0/g' \
-e 's/pV 119.2/V 120.5/g' \
-e 's/pW 121.3/W 122.2/g' \
-e 's/pY 120.3/Y 120.8/g' _ttemp_1 > _temp_1 

paste _temp_* | sed -e 's/ /      /g' | awk '{ print $2+$3+$4+$5+$6}' | sed -e 's/^-....8/9999/g' | awk '{ print $0}'>_n_corrections

 rm _*temp_*

### deal with CB chemical shift without CORRECTIONS
tr 'A-Z' 'a-z' < $1 | tr '[:punct:]' 'b' | tr '0-9' 'b' |tr '\n' 'b'| \
sed -e 's/[o|x|z|u|j|b| ]//g' \
-e 's/a/A 19.26\n/g' \
-e 's/c/C 28.34\n/g' \
-e 's/d/D 41.10\n/g' \
-e 's/e/E 29.90\n/g' \
-e 's/f/F 39.75\n/g' \
-e 's/g/G 9999\n/g' \
-e 's/h/H 29.00\n/g' \
-e 's/i/I 38.91\n/g' \
-e 's/k/K 33.10\n/g' \
-e 's/l/L 42.46\n/g' \
-e 's/m/M 32.94\n/g' \
-e 's/n/N 39.09\n/g' \
-e 's/p/P 32.22\n/g' \
-e 's/q/Q 29.53\n/g' \
-e 's/r/R 30.90\n/g' \
-e 's/s/S 64.06\n/g' \
-e 's/t/T 70.01\n/g' \
-e 's/v/V 32.82\n/g' \
-e 's/w/W 29.75\n/g' \
-e 's/y/Y 38.94\n/g' | awk '{gsub("n","\n");print}'|awk '{print $2}'|sed -e 's/N/ /'g > _cb_no_corrections

###
###  intergrate each corrected chemical shif file into one table
###
cat _coil | awk ' {print $1}' > _res_temp
num=1
i=`expr $i + 1`
while [ $num -lt $i ]
do
echo $num >> _number_temp
num=`expr $num + 1`
done

echo "Seq-dependent ppm corrections of CA,CO,HA,HN,N and no corrections of CB
num    res      HN       HA       CA      CB       CO      N
==============================================================="  > random_coil_chem_shifts.tbl
paste _number_temp _res_temp _hn_corrections _ha_corrections _ca_corrections _cb_no_corrections _co_corrections _n_corrections >> random_coil_chem_shifts.tbl
#
# special problems on mac OS X 
# always produces one more line in the end, so play a trick to remove it
# 2007, 07,11
i=`expr $i + 1`
cat  random_coil_chem_shifts.tbl |head -n $i >  _temp3
mv _temp3 random_coil_chem_shifts.tbl

# echo DONE! The output file is named as corrected_shifts.tbl

 rm _*correction* _coil _*temp*

